"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const types_1 = require("hint/dist/src/lib/types");
const load_json_file_1 = require("hint/dist/src/lib/utils/fs/load-json-file");
const schema_validator_1 = require("hint/dist/src/lib/utils/schema-validator");
class TypeScriptConfigParser extends types_1.Parser {
    constructor(engine) {
        super(engine, 'typescript-config');
        this.schema = load_json_file_1.default(path.join(__dirname, 'schema.json'));
        engine.on('fetch::end::*', this.parseTypeScript.bind(this));
    }
    async validateSchema(config, resource) {
        const validationResult = schema_validator_1.validate(this.schema, config);
        const valid = validationResult.valid;
        if (!valid) {
            const event = {
                errors: validationResult.errors,
                prettifiedErrors: validationResult.prettifiedErrors,
                resource
            };
            await this.engine.emitAsync(`parse::${this.name}::error::schema`, event);
        }
        return validationResult;
    }
    async parseTypeScript(fetchEnd) {
        const resource = fetchEnd.resource;
        const fileName = path.basename(resource);
        if (!fileName.match(/^tsconfig\.([^.]*\.)?json/gi)) {
            return;
        }
        const parseStart = { resource };
        await this.engine.emitAsync(`parse::${this.name}::start`, parseStart);
        let config;
        try {
            config = JSON.parse(fetchEnd.response.body.content);
            const originalConfig = lodash_1.cloneDeep(config);
            config = await this.finalConfig(config, resource);
            if (!config) {
                return;
            }
            const validationResult = await this.validateSchema(config, resource);
            if (!validationResult.valid) {
                return;
            }
            const event = {
                config: validationResult.data,
                originalConfig,
                resource
            };
            await this.engine.emitAsync(`parse::${this.name}::end`, event);
        }
        catch (err) {
            const errorEvent = {
                error: err,
                resource
            };
            await this.engine.emitAsync(`parse::${this.name}::error::json`, errorEvent);
        }
    }
}
exports.default = TypeScriptConfigParser;
